"""Script for running a tsunami inundation scenario for Hobart, TAS, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a tsunami wave generated by MOST.

Ole Nielsen and Duncan Gray, GA - 2005 and Nick Bartzis, GA - 2006
"""
#-------------------------------------------------------------------------------# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os
import time
from shutil import copy
from os import mkdir, access, F_OK
import sys

# Related major packages
from anuga.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.shallow_water.data_manager import convert_dem_from_ascii2netcdf, dem2pts
from anuga.geospatial_data.geospatial_data import *
from anuga.abstract_2d_finite_volumes.util import Screen_Catcher

# Application specific imports
import project                 # Definition of file names and polygons

#-------------------------------------------------------------------------------
# Copy scripts to time stamped output directory and capture screen
# output to file
#-------------------------------------------------------------------------------

# creates copy of code in output dir if dir doesn't exist
if access(project.outputtimedir,F_OK) == 0 :
    mkdir (project.outputtimedir)
copy (project.codedirname, project.outputtimedir + project.codename)
copy (project.codedir + 'run_hobart.py', project.outputtimedir + 'run_hobart.py')
print'output dir', project.outputtimedir

#normal screen output is stored in 
screen_output_name = project.outputtimedir + "screen_output.txt"
screen_error_name = project.outputtimedir + "screen_error.txt"

#used to catch screen output to file
sys.stdout = Screen_Catcher(screen_output_name)
#sys.stderr = Screen_Catcher(screen_output_name)
sys.stderr = Screen_Catcher(screen_error_name)

print 'USER:    ', project.user

#-------------------------------------------------------------------------------
# Preparation of topographic data
#
# Convert ASC 2 DEM 2 PTS using source data and store result in source data
#-------------------------------------------------------------------------------

# filenames
onshore_dem_name = project.onshore_dem_name
meshname = project.meshname+'.msh'
source_dir = project.boundarydir

copied_files = False

# creates DEM from asc data
convert_dem_from_ascii2netcdf(onshore_dem_name, use_cache=True, verbose=True)

#creates pts file for onshore DEM
dem2pts(onshore_dem_name, use_cache=True, verbose=True)

print 'local offshore data sets'
G1 = Geospatial_data(file_name = project.offshore_dem_name_local1 + '.xya')
#G2 = Geospatial_data(file_name = project.offshore_dem_name_local2 + '.xya')
G3 = Geospatial_data(file_name = project.offshore_dem_name_local3 + '.xya')
G4 = Geospatial_data(file_name = project.offshore_dem_name_local4 + '.xya')
print 'aho offshore data sets'
G5 = Geospatial_data(file_name = project.offshore_dem_name_aho1 + '.xya')
G6 = Geospatial_data(file_name = project.offshore_dem_name_aho2 + '.xya')
G7 = Geospatial_data(file_name = project.offshore_dem_name_aho3 + '.xya')
G8 = Geospatial_data(file_name = project.offshore_dem_name_aho4 + '.xya')
G9 = Geospatial_data(file_name = project.offshore_dem_name_aho5 + '.xya')
G10 = Geospatial_data(file_name = project.offshore_dem_name_aho6 + '.xya')
G11 = Geospatial_data(file_name = project.offshore_dem_name_aho7 + '.xya')
G12 = Geospatial_data(file_name = project.offshore_dem_name_aho8 + '.xya')
G13 = Geospatial_data(file_name = project.offshore_dem_name_aho9 + '.xya')
G14 = Geospatial_data(file_name = project.offshore_dem_name_aho10 + '.xya')
G15 = Geospatial_data(file_name = project.offshore_dem_name_aho11 + '.xya')
G16 = Geospatial_data(file_name = project.offshore_dem_name_aho12 + '.xya')
G17 = Geospatial_data(file_name = project.offshore_dem_name_aho13 + '.xya')
G18 = Geospatial_data(file_name = project.offshore_dem_name_aho14 + '.xya')
G19 = Geospatial_data(file_name = project.offshore_dem_name_aho15 + '.xya')
G20 = Geospatial_data(file_name = project.offshore_dem_name_aho16 + '.xya')
print 'local onshore dem'
G21 = Geospatial_data(file_name = project.onshore_dem_name + '.pts')
print 'coast'
G22 = Geospatial_data(file_name = project.coast_dem_name + '.xya')
print 'adding data sets'
G = G3+G4+G5+G6+G7+G8+G9+G10+G11+G12+G13+G14+G15+G16+G17+G18+G19+G20+G21+G22
G.export_points_file(project.combined_dem_name + '.pts')

#-------------------------------------------------------------------------------                                 
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#-------------------------------------------------------------------------------

from anuga.pmesh.mesh_interface import create_mesh_from_regions

# use 75 for onshore components (12.5m DEM)
island_res = 35000
hobart_res = 35000
peninsula_res = 35000
interior_regions = [[project.poly_hobart, hobart_res],
                    [project.poly_tasman_peninsula, peninsula_res],
                    [project.poly_bruny, island_res]]

print 'number of interior regions', len(interior_regions)

from caching import cache
_ = cache(create_mesh_from_regions,
          project.polyAll,
          {'boundary_tags': {'bottom': [0], 'right': [1],
                             'top': [2], 'left': [3]},
           'maximum_triangle_area': 250000,
           'filename': meshname,           
           'interior_regions': interior_regions},
          verbose = True, evaluate=True)


#-------------------------------------------------------------------------------                                 
# Setup computational domain
#-------------------------------------------------------------------------------                                 
domain = Domain(meshname, use_cache = False, verbose = True)

print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()
print domain.statistics()

domain.set_name(project.basename)
domain.set_datadir(project.outputtimedir)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
domain.set_minimum_sww_depth(0.01)
domain.set_store_vertices_uniquely(True)  # for writting to sww

#-------------------------------------------------------------------------------                                 
# Setup initial conditions
#-------------------------------------------------------------------------------

tide = 0.0

domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 

domain.set_quantity('elevation', 
#                    filename = project.onshore_dem_name + '.pts',
                    filename = project.combined_dem_name + '.pts',
#                    filename = project.offshore_dem_name + '.pts',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1
                    )

#-------------------------------------------------------------------------------                                 
# Setup boundary conditions (all reflective)
#-------------------------------------------------------------------------------
print 'start ferret2sww'
from anuga.shallow_water.data_manager import ferret2sww
'''
south = project.south 
north = project.north
west = project.west
east = project.east

#note only need to do when an SWW file for the MOST boundary doesn't exist
cache(ferret2sww,
      (source_dir + project.boundary_basename,
       source_dir + project.boundary_basename), 
#      (project.MOST_dir + project.boundary_basename,
#       source_dir + project.boundary_basename), 
      {'verbose': True,
# note didn't work with the below
#       'minlat': south - 1,
#       'maxlat': north + 1,
#       'minlon': west - 1,
#       'maxlon': east + 1,
       'minlat': south,
       'maxlat': north,
       'minlon': west,
       'maxlon': east,
#       'origin': project.mesh_origin,
       'origin': domain.geo_reference.get_origin(),
       'mean_stage': tide,
       'zscale': 1,                 #Enhance tsunami
       'fail_on_NaN': False,
       'inverted_bathymetry': True},
      #evaluate = True,
       verbose = True,
       dependencies = source_dir + project.boundary_basename + '.sww')

'''
print 'Available boundary tags', domain.get_boundary_tags()

#Bf = File_boundary(source_dir + project.boundary_basename + '.sww', 
#                    domain, verbose = True)
Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])


# 7 min square wave starting at 1 min, 6m high
Bw = Time_boundary(domain = domain,
                   f=lambda t: [(60<t<480)*6, 0, 0])

# for MOST BC
#domain.set_boundary( {'top': Bd, 'left': Bd,
#                      'bottom': Bf, 'right': Bf} )

# for testing
domain.set_boundary( {'top': Bd, 'left': Bd,
                      'bottom': Bd, 'right': Bw} )

#-------------------------------------------------------------------------------                                 
# Evolve system through time
#-------------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 240, finaltime = 7200): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'bottom')     

for t in domain.evolve(yieldstep = 120, finaltime = 12600
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'bottom')     
    
print 'That took %.2f seconds' %(time.time()-t0)

print 'finished'
